#!/usr/bin/env python
#
# $Id: WebAgent.py,v 1.10 2002/01/05 14:03:35 dhellmann Exp $
#

"""WebAgent Base Class

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: WebAgent.py,v $',
    'rcs_id'       : '$Id: WebAgent.py,v 1.10 2002/01/05 14:03:35 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <DougHellmann@bigfoot.com>',
    'project'      : 'CMFAgents',
    'created'      : 'Thu, 12-Jul-2001 10:44:08 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.10 $',
    'date'         : '$Date: 2002/01/05 14:03:35 $',
}

#
# Import system modules
#
import urllib
import sys
from formatter import NullFormatter
import htmllib
import string
import pprint
import re
import UserList

#
# Import Local modules
#
from CMFFeedApp import CMFFeedApp
import DateTime


#
# Module
#

date_pattern='(?P<month>\d+)/(?P<day>\d+)/(?P<year>\d+)'
time_pattern= '(?P<hour>\d+):(?P<minute>\d+):(?P<second>\d+)\s+(?P<ampm>(AM|PM))'
date_parts_splitter=re.compile('\s*' + date_pattern + '\s+' + time_pattern).match

def date_to_datetime(input_date):
    # Example:   4/30/2001 9:18:59 AM
    match = date_parts_splitter(input_date)
    if not match:
        raise ValueError(input_date)
    parts = match.groupdict()
    dt_values = map( lambda x: int(x),
                     (parts['year'],
                      parts['month'],
                      parts['day'],
                      parts['hour'],
                      parts['minute'],
                      parts['second'],
                      )
                     )
    
    if (string.lower(parts['ampm']) == 'pm') and (dt_values[3] < 12):
        dt_values[3] = dt_values[3] + 12

    dt_values.append('EST')
    dt_values = tuple(dt_values)
    date_time = apply(DateTime.DateTime, dt_values)
    return date_time

def time_difference(start, finish):
    #
    # Define the epoch minus seconds.
    #
    epoch = (1969, 12, 31, 0, 0)
    #
    # Get the pieces of the date value but ignore
    # the timezone and seconds.
    #
    finish = finish.parts()[:-2]
    start = start.parts()[:-2]
    time_diff = map( lambda x, y: y-x,
                     start,
                     finish
                     )
    years, months, days, hours, minutes = time_diff
    if minutes < 0:
        hours = hours - 1
        minutes = minutes + 60
    if hours < 0:
        days = days - 1
        hours = hours + 24
    if days < 0:
        months = months - 1
        # rough estimate / average
        days = days + 30
    if months < 0:
        years = years - 1
        months = months + 12
        
    time_diff = years, months, days, hours, minutes
        
    return time_diff

def str_time_diff(args):
    part_names = ('year',
                  'month',
                  'day',
                  'hour',
                  'minute',
                  )
    rep = ''
    for val, name in map(None, args, part_names):
        if val:
            if val > 1:
                plural='s'
            else:
                plural=''
            rep = '%s %d %s%s' % (rep, val, name, plural)
    return string.strip(rep)

class SearchResultItem:
    def __init__(self, url, title='', description=''):
        self.url = url
        self.setTitle(title)
        self.setDescription(description)
        return

    def setTitle(self, title):
        self.title = title
        return

    def setDescription(self, description):
        self.description = description
        return

    def __str__(self):
        return '%s\n\t%s\n\n\t%s' % (self.title, self.url, self.description)
    

class SearchResultsList(htmllib.HTMLParser, UserList.UserList):
    """Base class for parser that knows about search engine output.

    This parser (based on HTMLParser from htmllib) knows how to
    extract information from a summary report generated by a search
    engine.  The specific knowledge of a search engine's output must
    be provided by the subclass.  This parser provides the basic
    services necessary to construct such a tool.  
    
    """

    def __init__(self):
        "Create a SearchResultsList."
        UserList.UserList.__init__(self)
        self.field_names = {}
        self.current_record = None
        self.trace = 0
        
        formatter = NullFormatter()
        htmllib.HTMLParser.__init__(self, formatter)

        self._continuation_links = []
        return

    def tuplesToDict(self, attrs):
        """Convert a sequence of tuples to a dictionary.

        The sequence should contain ('name', 'value') pairs.  The
        'name' value will be used as the key of the dictionary, and
        the 'value' will be stored as the value.  Duplicate 'name'
        values are not detected.
        """
        d = {}
        for name, value in attrs:
            d[name] = value
        return d

    def __str__(self):
        return '\n\n--------------------\n\n'.join([ str(x) for x in self ])

    def getContinuationList(self):
        return self._continuation_links
    
    
class WebAgent(CMFFeedApp):

    process_all = 1

    #
    # Static settings to provide access to web input
    #

    default_config_values = {}
    default_config_values.update(CMFFeedApp.default_config_values)
    default_config_values.update({
        'URL':'',
        'COOKIES':[],
        })

    def getURL(self, searchTerms):
        base_url = 'http://%s%s' % (self.SERVER, self.URL)
        url_arguments = self.getURLArguments(searchTerms)
        self.statusMessage('URL arguments: %s' % str(url_arguments), 3)
        params = urllib.urlencode( url_arguments )
        if params:
            full_url='%s?%s' % (base_url, params)
        else:
            full_url=base_url
        return full_url

    def optionHandler_a(self):
        "Process all pages."
        self.process_all = 1
        return

    def optionHandler_A(self):
        "Do not process all pages."
        self.process_all = 0
        return

    def retrieveUrl(self, urlToFetch):
        "Retrieve the data to be processed."
        #urlOpener.auth_cache['TeamTrack@teamtrack.zapmedia.com'] = (self.report_user,
        #                                                            self.report_password)
        url_opener = urllib.FancyURLopener()
        for cookie in self.COOKIES:
            url_opener.addheader('Cookie', '%s=%s' % cookie)
        self.statusMessage( 'Retrieving data from server (%s)...' % urlToFetch, 2)
        tmpfile, dataSource  = url_opener.retrieve(urlToFetch)
        data = open(tmpfile, 'rt').read()
        self.statusMessage('Retrieved [[%s]]' % data, 3)
        for bad_char in ( 0x99, 0x92 ):
            data.replace(chr(bad_char), '')
        return data

    def processOneResult(self, parser, hit, folder):
        "Override this method to handle one of the results in the result set."
        raise RuntimeError('%s.processOneResult not implemented.' \
                           % self.__class__.__name__)

    def processResults(self, parser, folder):
        """Call self.processOneResult() for eac item in the result set.

        This method can usually be left alone, but it is here so that
        if a more efficient manner of processing an entire result set
        is available it can be used.
        """
        for hit in parser:
            self.processOneResult(parser, hit, folder)
        return

    def createResulstList(self):
        "Returns a new, empty, result set."
        return SearchResultsList()
            
    def loadDataForFolder(self, folder):
        """Extract search terms from the title of the folder, and run the search.
        """
        self.statusMessage('%s:' % folder.id, 2)
        #
        # Record of what we've seen
        #
        processed_urls = []
        
        #
        # Record of what we need to look at
        #
        url_queue = []
        
        #
        # Set up queue of continuation pages to fetch
        #
        default_url = self.getURL(folder)
        url_queue.append(default_url)

        while url_queue:

            #
            # Queue management
            #
            url_to_fetch = url_queue[0]
            url_queue = url_queue[1:]
            processed_urls.append(url_to_fetch)

            #print 'URLS LEFT:' ,
            #pprint.pprint(url_queue)
            #print 'URLS PROCESSED:',
            #pprint.pprint(processed_urls)
            #print 'CURRENT:', url_to_fetch

            #
            # Fetch the data
            #
            try:
                data = self.retrieveUrl(url_to_fetch)
            except KeyboardInterrupt:
                raise
            except Exception, msg:
                self.statusMessage('Error fetching %s' % url_to_fetch)
                self.statusMessage('ERROR: %s' % str(msg))
                raise
                #continue

            #
            # Parse the data
            #
            try:
                parser = self.createResultsList()
                parser.feed(data)
                parser.close()
            except KeyboardInterrupt:
                raise
            except Exception, msg:
                self.statusMessage('Error parsing %s' % url_to_fetch)
                self.statusMessage('ERROR: %s' % str(msg))

            #
            # Process results
            #
            try:
                self.processResults(parser, folder)
            except KeyboardInterrupt:
                raise
            except Exception, msg:
                self.statusMessage('Error processing %s' % url_to_fetch)
                self.statusMessage('ERROR: %s' % str(msg))

            #
            # Add additional pages to queue
            #
            if self.process_all:
                for continuation_page in parser.getContinuationList():
                    continue_url = 'http://%s%s' % (self.SERVER,
                                                    continuation_page)
                    if ( (continue_url not in url_queue)
                         and
                         (continue_url not in processed_urls)
                         ):
                        url_queue.append(continue_url)

        self.statusMessage('Processed %d pages of results.' % len(processed_urls), 2)
        return
