#!/usr/local/bin/python
#
# $Id: agent.py,v 1.1 2002/01/08 18:22:03 dhellmann Exp $
#

"""CMF Web Agent

  Retrieves a search result set and load each item into a CMF Portal.
"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: agent.py,v $',
    'rcs_id'       : '$Id: agent.py,v 1.1 2002/01/08 18:22:03 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'CMFAgents',
    'created'      : 'Wed, 13-Jun-2001 07:38:43 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.1 $',
    'date'         : '$Date: 2002/01/08 18:22:03 $',
}

#
# Import system modules
#

#
# Import Local modules
#
from google_parser import GoogleSearchResultsList
from WebAgent import WebAgent, SearchResultsList, SearchResultItem

#
# Module
#

class CMFWebAgent(WebAgent):

    default_config_values = {}
    default_config_values.update(WebAgent.default_config_values)
    default_config_values.update({
        'SERVER':'www.google.com',
        'URL':'/search',
        'COOKIES':[ ('PREF',
                     'ID=2596116a018b7b68:LD=en:NR=100:NW=1:TM=993551980:LM=996694990'),
                    ],
        'ignore_titles':[ ],
        'lang':'',
        'max_title_len':30,
        })

    def getURLArguments(self, folder):
        search_terms = folder.description.strip()
        arguments = { 'start':0,
                      'filter':0,
                      }
        if self.lang:
            arguments['lr']=self.lang
                                  
        if search_terms:
            arguments['q'] = search_terms
        else:
            self.showHelp('No search terms were specified.  Use -s or -h.')
            raise self.HelpRequested
        return arguments

    def createResultsList(self):
        return GoogleSearchResultsList()

    def processResults(self, parser, folder):
        #
        # Reverse to put the least-likely matches
        # first.  This, in turn, makes the creation
        # times work out so that the most likely matches
        # are created last and appear closer to the
        # top of the "current news" list.
        #
        parser.reverse()
        return WebAgent.processResults(self, parser, folder)
        

    def processOneResult(self, parser, hit, folder):
        
        url = hit.url
        id = self.urlToId(url)
        title = hit.title.strip()

        # Possible short-cut
        if title in self.ignore_titles:
            self.statusMessage('Skipping %s' % id, 2)
            return

        # start the description with the title, in case it
        # is truncated later
        description = '<h4>%s</h4>' % title

        # Clean up the title.
        for tag in ( 'b', 'i', 'u', 'font' ):
            title = title.replace('<%s>' % tag, '')
            title = title.replace('</%s>' % tag, '')
        if len(title) > self.max_title_len:
            title = '%s ...' % title[:self.max_title_len]

        # Finish off the description, and include the url
        description = description + ('<p>%s</p>' % hit.description)
        description = description + ('<p><b>Remote Page:</b> %s</p>' % url)

        # Try to store the article
        self.addNewsItem(folder, id, title, url, description)
        return


#
# Main
#
if __name__ == '__main__':
    try:
        CMFWebAgent().run()
    except CMFWebAgent.HelpRequested:
        pass
    
