#!/usr/bin/env python
#
# $Id: google_parser.py,v 1.3 2002/01/01 14:11:00 dhellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Parse Google search results.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: google_parser.py,v $',
    'rcs_id'       : '$Id: google_parser.py,v 1.3 2002/01/01 14:11:00 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'WebAgents',
    'created'      : 'Mon, 31-Dec-2001 08:20:34 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.3 $',
    'date'         : '$Date: 2002/01/01 14:11:00 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#
import pprint
import re
import string
import sys

#
# Import Local modules
#
from WebAgent import SearchResultsList, SearchResultItem


#
# Module
#

class GoogleSearchResultsList(SearchResultsList):

    parseResultExpr = re.compile(
        '<p><a href="?(?P<link>[^" >]*)[^>]*>(?P<title>.*?)</a>.*?<br>.*?<font[^>]*>(?P<description>.*)'
        )
    
    parseNextPageExpr = re.compile('<td><a href="?(?P<link>[^" ]*)"?.*><img src=')

    def feed(self, text):
        lines = text.split('\n')
        parser = self.parseResultExpr
        for line in lines:
            line = line.strip()
            if not line:
                continue
            if line[:3] == '<p>':
                # Line with link and description
                match = parser.search(line)
                #match = parser.match(line)
                if match:
                    params = match.groupdict()
                    #print 'MATCHED:',
                    #pprint.pprint(params)
                    #print 
                    new_item = SearchResultItem(params['link'],
                                                params['title'],
                                                params['description'])
                    self.append(new_item)
            elif line[:4] == '<td>':
                # Possibly a line with "next" page of results
                match = self.parseNextPageExpr.search(line)
                if match:
                    link = match.groupdict()['link']
                    self._continuation_links.append(link)
        return


    
class HTMLParser_GoogleSearchResultsList(SearchResultsList):

    def __init__(self):
        SearchResultsList.__init__(self)

        self._buffers = {}
        self._current_buffer_stack = ()
        self._accumulate_data = 0
        
        self._startIgnoring()
        self._inProgress = None
        return

    def _startIgnoring(self):
        self.ignore_inputs = 1
        return

    def _stopIgnoring(self):
        self.ignore_inputs = 0
        return

    def _startBufferingData(self, bufferName):
        self._accumulate_data = self._accumulate_data + 1
        self._buffers[bufferName] = ''
        self._current_buffer_stack = (bufferName, self._current_buffer_stack)
        #print 'stack: %s' % str(self._current_buffer_stack)
        return

    def _stopBufferingData(self, bufferName):
        data = self._buffers[bufferName]
        self._accumulate_data = self._accumulate_data - 1
        
        if self._current_buffer_stack:
            if self._current_buffer_stack[0] == bufferName:
                # can pop because we're finishing
                # what we think
                old_buffer_name, self._current_buffer_stack = self._current_buffer_stack
            else:
                # popping something that isn't
                # there on the stack
                old_buffer_name = ''
        else:
            old_buffer_name = ''
            
        if old_buffer_name != bufferName:
            self._buffers[bufferName] = ''
        return data

    def start_table(self, params):
        self._startIgnoring()
        return
    
    def end_table(self):
        self._stopIgnoring()
        return

    def start_a(self, params):
        if not self.ignore_inputs:
            params = self.tuplesToDict(params)
            #print 'start_a(%s)' % str(params)
            if params.get('class', None) == '1':
                # sub-link, ignore
                self._startBufferingData('sublink')
            elif params.get('href', '/')[0] == '/':
                # local link to google site
                self._startBufferingData('google_link')
            else:
                self._inProgress = SearchResultItem(params['href'])
                self._startBufferingData('title')
        return

    def end_a(self):
        if not self.ignore_inputs:

            if ( self._current_buffer_stack
                 and
                 (self._current_buffer_stack[0] == 'title')
                 ):
                title = self._stopBufferingData('title')
            
                save = 1
                if title in ('Cached', 'Similar pages', 'AltaVista',
                             'Excite', 'Google Groups (Deja)',
                             'HotBot', 'Lycos', 'Yahoo!'):
                    save = 0
                elif title[:17] == 'More results from':
                    save = 0

                if save:
                    self._inProgress.setTitle(title)
                    self.append(self._inProgress)
                #print 'Ending with (%s)' % title

            elif self._current_buffer_stack:
                toss = self._stopBufferingData(self._current_buffer_stack[0])

            else:
                print 'NO STACK TO POP'
        return

    def start_font(self, params):
        if not self.ignore_inputs:
            params = self.tuplesToDict(params)
            if params.get('size', None) == '-1':
                self._startBufferingData('description')
        pass

    def end_font(self):
        if not self.ignore_inputs:
            description = self._stopBufferingData('description')
            #print 'end_font() description="%s"' % description
            #print '***', self._inProgress
            if self._inProgress:
                self._inProgress.setDescription(description)
                #self._inProgress = None
            else:
                print 'NOT IN PROGRESS (%s)' % description
            #print
        pass

    def start_blcokquote(self, params):
        if not self.ignore_inputs:
            self._startBufferingData('description')
        return

    def end_blockquote(self):
        if not self.ignore_inputs:
            description = self._stopBufferingData('description')
            if self._inProgress:
                self._inProgress.setDescription(description)
        return

    def handle_data(self, data):
        if self._accumulate_data:
            try:
                buffer_name = self._current_buffer_stack[0]
                #print 'handle_data[%s](%s)' % (buffer_name, data)
                self._buffers[buffer_name] = self._buffers[buffer_name] + data
            except IndexError:
                #print 'handle_data[%s](%s)' % ('UNKNOWN', data)
                pass
        return



if __name__ == '__main__':
    data = open('sample_google_output.html', 'rt').read()
    
    parser = GoogleSearchResultsList()
    parser.feed(data)
    parser.close()

    print str(parser)
